import java.util.*;

/**
 * Essa classe  uma classe auxiliar que serve para manipulacao de dados<br>
 * contendo datas no formato dd/MM/aaaa
 *
 *
 */
public class Data {
   private int mes;
   private int dia;
   private int ano;

   /**
    * Construtor da classe Data
    *
    * Nao recebe nenhum parametro e cria uma data com dia mes e ano igual a 1
    *
    */

   public Data() {
      mes = 1;
      dia = 1;
      ano = 1;
   }

   /**
    * Construtor da classe Data
    * @param d Dia
    * @param m Mes
    * @param a Ano
    */

   public Data(int d, int m, int a) throws NumeroInvalidoException {
      mes = 1;
      dia = 1;
      ano = 1;
      this.setAno(a);
      this.setMes(m);
      this.setDia(d);
   }

   /**
    * Metodo que retorna o ano
    * @return O ano da data
    */

   public int getAno() {
      return this.ano;
   }

   /**
    * Metodo que retorna o mes
    * @return O mes da data
    */

   public int getMes() {
      return this.mes;
   }

   /**
    * Metodo que retorna o dia
    * @return O dia da data
    */

   public int getDia() {
      return this.dia;
   }

   /**
    * Metodo que seta o ano
    * @param a O ano da data, que deve ser maior que 0
    */

   public void setAno(int a) throws NumeroInvalidoException {
      if (a < 0) {
         this.ano = 2002;
         this.setDia(dia);
         throw new NumeroInvalidoException(
           "Ano invalido. Apenas numeros positivos");
      }
      else {
         this.ano = a;
         this.setDia(dia);
      }
   }

   /**
    * Metodo que seta o mes
    * @param m O mes da data, que fica entre 1 e 12
    */

   public void setMes(int m) throws NumeroInvalidoException {
      if (m > 12 || m < 1) {
         throw new NumeroInvalidoException(
           "Mes invalido. Numeros entre 1 e 12");
      }
      else {
         this.mes = m;
      }
   }

   /**
    * Metodo que seta o dia
    * @param d Dia da data, que pode ser de 1 a 31 dependendo do mes
    */

   public void setDia(int d) throws NumeroInvalidoException {
      int m = mes;

      if (d > 31 || d < 1) {
         this.dia = 1;
         throw new NumeroInvalidoException("Dia invalido. Dia = 1");
      }
      else {
         if (m == 1 || m == 3 || m == 5 || m == 7 || m == 8 || m == 10
             || m == 12) {
            this.dia = d;
         }
         if (m == 4 || m == 6 || m == 9 || m == 11) {
            if (d == 31) {
               this.dia = 1;
               throw new NumeroInvalidoException("Dia invalido. Dia = 1");
            }
            else {
               this.dia = d;
            }
         }

         if (m == 2) {
            if (ano % 4 == 0) {
               if (d > 29) {
                  this.dia = 1;
                  throw new NumeroInvalidoException(
                    "Dia invalido. Dia = 1");
               }
               else {
                  this.dia = d;
               }
            }
            else {
               if (d > 28) {
                  this.dia = 1;
                  throw new NumeroInvalidoException(
                    "Dia invalido. Dia = 1");
               }
               else {
                  this.dia = d;
               }
            }
         }
      }
   }

   /**
    * Metodo que atualiza a data, dando outra como parametro
    * @param a Data para atualizar a antiga
    */

   public void atualizar(Data a) throws NumeroInvalidoException {
      if (a != null) {
         this.setDia(a.getDia());
         this.setMes(a.getMes());
         this.setAno(a.getAno());
      }
      else {
         throw new NumeroInvalidoException("Data nao atualizada");
      }
   }

   /**
    * Metodo que diz se uma data  igual aquela passada como parametro
    * @param a Data a ser comparada
    * @return true se as datas sao iguais, false se nao sao iguais
    */

   public boolean equals(Data a) throws ElementoNuloException {
      boolean r = false;
      if (a != null) {
         if (this.getAno() == a.getAno() && this.getMes() == a.getMes() &&
             this.getDia() == a.getDia()) {
            r = true;
         }
      }
      else {
         throw new ElementoNuloException("Data nula");
      }
      return r;
   }

   /**
    * Metodo que verifica se a data que chama o metodo  anterior a passada como parametro
    * @param a Data a ser comparada
    * @return true se a data for anterior a data passada como parametro, false se nao
    */

   public boolean anterior(Data a) throws ElementoNuloException {
      boolean r = true;
      if (a != null) {
         if (this.equals(a)) {
            r = false;
         }
         if (this.getAno() > a.getAno()) {
            r = false;
         }
         else {

            if (this.getAno() == a.getAno()) {
               if (this.getMes() > a.getMes()) {
                  r = false;
               }
               else {
                  if (this.getMes() == a.getMes()) {
                     if (this.getDia() > a.getDia()) {
                        r = false;
                     }
                  }
               }
            }
         }
      }
      else {
         throw new ElementoNuloException("Data nula");
      }
      return r;
   }

   /**
    * Metodo que verifica se a data que chama o metodo  posterior a passada como parametro
    * @param a Data a ser comparada
    * @return true se a data for posterior a data passada como parametro, false se nao
    */

   public boolean posterior(Data a) throws ElementoNuloException {
      boolean r = true;
      if (a != null) {
         if (this.equals(a)) {
            r = false;
         }
         if (this.anterior(a)) {
            r = false;
         }
      }
      return r;
   }

   /**
    * Metodo que le uma data do teclado, le o dia, o mes, depois o ano e retorna um objeto data
    * @return A Data lida do teclado
    */
//no useful..
   /**public static Data readData()
       throws NumeroInvalidoException {
       Data d1 = new Data();
       c.println("-> Digite o dia: ");
       d1.setDia(c.readInt());
       c.println("-> Digite o mes: ");
       d1.setMes(c.readInt());
       c.println("-> Digite o ano: ");
       d1.setAno(c.readInt());
       return d1;
        }*/

   /**
    * Metodo que transforma da data passada como parametros em dias decorridos desde 1/1/1
    * @param a Data transformada em dias
    * @return um inteiro que representa o numero de dias passados desde o 1/1/1
    */

   public static int transDias(Data a) {
      int diasd = 0;
      for (int i = 1; i <= a.getAno(); i++) {
         if (i % 4 == 0) {
            diasd = diasd + 366;
         }
         else {
            diasd = diasd + 365;
         }
      }
      for (int i = 1; i <= a.getMes(); i++) {
         switch (i) {
            case 1:
            case 3:
            case 5:
            case 7:
            case 8:
            case 10:
            case 12:
               diasd = diasd + 31;
               break;
            case 4:
            case 6:
            case 9:
            case 11:
               diasd = diasd + 30;
               break;
            case 2:
               if (a.getAno() % 4 == 0) {
                  diasd = diasd + 29;
               }
               else {
                  diasd = diasd + 28;
               }
               break;
         }
      }
      diasd = diasd + a.getDia();
      return diasd;
   }

   /**
    * Metodo que retorna o numero de dias decorridos entre duas datas
    * @param a Data a ser comparada
      * @return um inteiro positivo que representa o numero de dias entre as datas
    */

   public int getDistanciaDias(Data d) {
      int diasd1 = 0;
      int diasd2 = 0;
      int diferenca = 0;

      diasd1 = Data.transDias(this);
      diasd2 = Data.transDias(d);

      diferenca = diasd1 - diasd2;
      if (diferenca < 0) {
         diferenca = -diferenca;
      }
      return diferenca;
   }

   /**
    * Metodo que retorna a representacao de uma data como uma String
    * @return uma String que representa o objeto data
    */

   public String toString() {
      String ret = "" + this.getDia() + "/" + this.getMes() + "/"
        + this.getAno() +
        "";
      return ret;
   }

   public String getData() {
      String ret = "" + this.getDia() + "/" + this.getMes() + "/"
        + this.getAno() +
        "";
      return ret;
   }

   /**
    * Metodo que transforma uma String num objeto Data
    * @param dt String no formato dd/MM/aaaa a ser transformada num objeto data
      * @return um objeto do tipo Data, com dia mes e ano iguais ao da string passada,
    *         ou null se a String passada nao estive no formato
    */

   public static Data transData(String dt) throws NumeroInvalidoException,
     FormatoInvalidoException {

      Data ddd = new Data();

      if (dt.length() == 10) {
         String day = dt.substring(0, 2);
         String month = dt.substring(3, 5);
         String year = dt.substring(6, 10);

         ddd.setDia(Integer.parseInt(day));
         ddd.setMes(Integer.parseInt(month));
         ddd.setAno(Integer.parseInt(year));
      }
      else {
         throw new FormatoInvalidoException("Formato de data invalido",
                                            "dd/MM/aaaa");
      }
      return ddd;
   }

   /**
    * Pega a data atual do sistema
    * @return A data atual do sistema
    */
   public static Data atual() {
      Data d = null;
      Date data = new Date(System.currentTimeMillis());
      Calendar cal = new GregorianCalendar();
      cal.setTime(data);
      try {
         d = new Data(cal.get(Calendar.DATE), cal.get(Calendar.MONTH) + 1,
                      cal.get(Calendar.YEAR));
      }
      catch (NumeroInvalidoException e) {}
      return d;
   }
}