import java.util.*;

public class Hora {
   private int hora;
   private int minuto;

   /**
    * Construtor sem parametro
    *
    *  Inicializa a hora e minutos igual a 0.
    */
   public Hora() {
      this.hora = 0;
      this.minuto = 0;
   }

   /**
    * Construtor com parametro da classe Hora
    *
    * @param hora      do tipo integer
    * @param minuto    do tipo integer
    *
    * @exception       FormatoInvalidoException     formato de hora invalido,
    *                                               formato de minuto invalido
    */
   public Hora(int hora, int minuto) {
      this.setHora(hora);
      this.setMinuto(minuto);
   }

   /**
    * Metodo que verifica se a hora que chama metodo e anterior a
    *  hora fornecida como parametro
    *
    * @param hora  do tipo Hora
    *
    * @return      true se a hora for anterior, se nao retorna false
    */
   public boolean anterior(Hora h) {
      boolean ant;
      if ( (this.hora < h.hora) | ( (this.hora == h.hora) &&
                                   (this.minuto < h.minuto))) {
         ant = true;
      }
      else {
         ant = false;
      }
      return ant;
   }

   /**
    * Metodo que verifica se a hora que chama metodo e posterior a
    *  hora fornecida como parametro
    *
    * @param hora  do tipo Hora
    *
    * @return      true se a hora for posterior, se nao retorna false
    */
   public boolean posterior(Hora h) {
      boolean post;
      if ( (this.hora > h.hora) || ( (this.hora == h.hora) &&
                                    (this.minuto > h.minuto))) {
         post = true;
      }
      else {
         post = false;
      }
      return post;
   }

   /**
    * Metodo que verifica se a hora que chama metodo e igual a
    *  hora fornecida como parametro
    *
    * @param hora  do tipo Hora
    *
    * @return      true se as horas forem iguais e false se nao forem
    */
   public boolean equals(Hora h) {
      return (this.hora == h.hora && this.minuto == h.minuto);
   }

   /**
    * Metodo que atualiza a hora (hora e minuto)
    *
    * @param hora  do tipo Hora
    *
    * @exception       ElementoNuloException     hora nula
    * @exception       FormatoInvalidoException     hora invalida
    */
   public void atualizar(Hora h) {
      if (h != null) {
         if (h.hora != this.hora && h.minuto != this.minuto) {
            this.setHora(h.hora);
            this.setMinuto(h.minuto);
         }
      }
   }

   /**
    * Metodo que retorna a hora
    *
    * @return hora  do tipo integer
    */
   public int getHora() {
      return this.hora;
   }

   /**
    * Metodo que modifica a hora
    *
    * @param hora  do tipo integer
    *
    * @exception       FormatoInvalidoException     formato de hora invalido
    */
   public void setHora(int hora) {
      if ( (hora <= 23) && (hora >= 0)) {
         this.hora = hora;
      }
      else {
         this.hora = 0;
      }

   }

   /**
    * Metodo que retorna o minuto
    *
    * @return minuto  do tipo integer
    */
   public int getMinuto() {
      return this.minuto;
   }

   /**
    * Metodo que modifica o minuto
    *
    * @param minuto  do tipo integer
    *
    * @exception       FormatoInvalidoException     formato de minuto invalido
    */
   public void setMinuto(int min) {
      if ( (min >= 0) && (min < 60)) {
         this.minuto = min;
      }
      else {
         this.minuto = 0;
      }
   }

   /***
    * Metodo usado para visualizar a hora
    *
    * @return hora   do tipo String
    */
   public String mostrarHora() {
      String mostrar = this.getHora() + ":" + this.getMinuto();
      return mostrar;
   }

   public String toString() {
      String mostrar = this.getHora() + ":" + this.getMinuto();
      return mostrar;
   }

   public static Hora transHora(String h) {
      Hora horaz = new Hora();
      if (h.length() == 5) {
         horaz.setHora(Integer.parseInt(h.substring(0, 2)));
         horaz.setMinuto(Integer.parseInt(h.substring(3, 5)));
      }
      return horaz;
   }

   public static Hora atual() {
      Date data = new Date(System.currentTimeMillis());
      Calendar cal = new GregorianCalendar();
      cal.setTime(data);
      return new Hora(cal.get(Calendar.HOUR), cal.get(Calendar.MINUTE));
   }

}
